#!/bin/bash

# define main function
MAIN_INSTALL="main_install"
MAIN_UPGRADE="main_upgrade"
MAIN_UNINST="main_uninst"
# define each page's name
PAGE_MARIADB5="page_mariadb5"
PAGE_MARIADB10="page_mariadb10"
PAGE_RESTORE="page_restore"
PAGE_UPGRADE_RESTRICTED="page_upgrade_restricted"
PAGE_DB="page_db"
PAGE_UNINST="page_uninst"
DEFAULT="enu"
# set out file name
OUTPUT_INSTALL="install_uifile"
OUTPUT_UPGRADE="upgrade_uifile"
OUTPUT_UNINST="uninstall_uifile"
INIT_DB_NAME="" # need to be overwrited
INIT_DB_USER="" # need to be overwrited
INFO_FILE="" # need to be overwrited
BACKUP_PATH="" # need to be overwrited
# here to define what page is necessary for each stage: install, upgrade, and uninstall
PAGE_INST="$PAGE_MARIADB5 $PAGE_RESTORE $PAGE_MARIADB10 $PAGE_DB $MAIN_INSTALL"
PAGE_UPGRADE="$PAGE_UPGRADE_RESTRICTED $PAGE_MARIADB5 $PAGE_MARIADB10 $PAGE_DB $MAIN_UPGRADE"
PAGE_UNINST="$PAGE_UNINST $MAIN_UNINST"
MIGRATE_DB_VERSION="-1"

MainCreate()
{
	local output="$1"
	local pages="$2"
	local main_function="$3"
	local lan
	for dir in lang/*; do
		lan=$(basename "$dir")
		cat "wizard_common" "wizard_customized" > "${output}_${lan}.sh"
		CustomWizCreate "${output}_${lan}.sh"
		cat "lang/$lan" $pages "$main_function" >> "${output}_${lan}.sh"
	done
	chmod +x "${output}"*.sh
	cp -a "${output}_${DEFAULT}.sh" "${output}.sh"
}

IsUpgradable()
{
	true
}

NeedMigrateDB()
{
	local version="$1"
	if [ -z "$version" ] || [ "$version" -le "$MIGRATE_DB_VERSION" ]; then
		return 0
	fi
	return 1
}

CustomHasRunWebsiteSetup()
{
	local path="$1"
	[ -s "$path/$CONF_FILE" ]
}

CustomWizCreate()
{
	:
}

CustomSetBackupPath()
{
	BACKUP_PATH="$(CustomGetBackupPkgPath)"
}

CustomMainInstall()
{
	:
}

CustomGetValueFromMetaFile()
{
	local key="$1"
	local backup_info="/var/packages/.$PKG_DIR.conf"
	local syno_conf="/var/packages/$PKG_NAME/synology.conf"
	local record_file=""

	if [ -s "$INFO_FILE" ]; then
		record_file="$INFO_FILE"
	elif [ -s "$backup_info" ]; then
		record_file="$backup_info"
	elif [ -s "$syno_conf" ]; then
		record_file="$syno_conf"
	fi
	get_key_value "$record_file" "$key"
}

GetPreviousPkgBuildNumber()
{
	local old_pkg_ver="$SYNOPKG_OLD_PKGVER"
	if [ -z "${old_pkg_ver}" ]; then
		# SYNOPKG_OLD_PKGVER does not exist in un-install stage of upgrade
		old_pkg_ver="$SYNOPKG_PKGVER"
	fi

	local old_build_number=$(echo "$old_pkg_ver" | cut -d- -f2)
	echo "$old_build_number"
}

CustomGetBackupPrefix()
{
	local backup_prefix="/$(readlink /var/services/web | cut -d/ -f2)/@appstore/.$PKG_DIR"
	echo "$backup_prefix"
}

CustomGetBackupPkgPath()
{
	local backup_pkg_path="$(CustomGetBackupPrefix)/$PKG_DIR"
	echo "$backup_pkg_path"
}

# we define pages in fuction not in global variables,
# because functions in wizard steps will be merged after calling MainCreate.
CustomSetPages()
{
	NEW_INSTALL_PAGES="$(PageM10),$(PageDB)"
	RESTORE_PAGES="$(PageRestore),$(PageM10),$(PageDB)"
	MIGRATE_PAGES="$(PageM5),$(PageRestore),$(PageM10),$(PageDB)"
}

CustomParseDBConf()
{
	:
}

CustomGetDBName()
{
	:
}

CustomGetDBUser()
{
	:
}

CustomGetDBPass()
{
	:
}

CustomPageActivate()
{
	:
}

quote_json() {
	sed -e 's|\\|\\\\|g' -e 's|\"|\\\"|g'
}

#!/bin/bash
# Copyright (c) 2000-2017 Synology Inc. All rights reserved.

PKG_NAME="Joomla"
PKG_DIR="joomla"
WEBSITE_ROOT="/var/services/web/$PKG_DIR"
CONF_FILE="configuration.php"
MIGRATE_DB_VERSION="0154"
INIT_DB_NAME="joomla"
INIT_DB_USER="joomla_user"
INFO_FILE="/usr/syno/etc/packages/$PKG_NAME/$PKG_DIR.conf"


CustomGetBackupPkgPath()
{
	local backup_prefix=$(CustomGetBackupPrefix)
	local backup_pkg_path="$(CustomGetBackupPrefix)/joomla"
	if [ ! -d "$backup_pkg_path" ]; then
		backup_pkg_path=$(dirname "$(ls -t "$backup_prefix"/*/configuration.php | head -1)")
	fi
	echo "$backup_pkg_path"
}

CustomParseDBConf()
{
	local path="$1"
	local db_info="$2"
	local grep_info="$3"
	local info_output=$(CustomGetValueFromMetaFile "$db_info")

	if [ -z "$info_output" ] && [ -s "$path/$CONF_FILE" ]; then
		info_output=$(grep "public \$$grep_info =" "$path/$CONF_FILE" | cut -d\' -f2)
	fi

	echo "$info_output"
}

CustomGetDBName()
{
	CustomParseDBConf "$1" "db_name" "db"
}

CustomGetDBUser()
{
	CustomParseDBConf "$1" "db_user" "user"
}

CustomGetDBPass()
{
	CustomParseDBConf "$1" "db_pass" "password"
}
wizard_db_settings="Joomla 데이터베이스 설정"
wizard_db_name_desc="Please enter database name for Joomla."
wizard_db_name_label="데이터베이스 이름"
wizard_m10_desc="Joomla 데이터베이스 설정을 진행하려면 MariaDB 10의 관리자 자격 증명을 입력하십시오."
wizard_admin_acc="계정"
wizard_admin_pass="패스워드"
wizard_set_db_desc="기존/새 데이터를 사용하려면 Joomla에 대한 새 독점 데이터베이스 계정을 만드십시오."
wizard_set_data_title="설정 Joomla"
wizard_found_backup="데이터를 가져올 방법을 선택하십시오."
wizard_decide_restore="Joomla 데이터베이스가 이미 있습니다.<br>다음 작업 중 하나를 선택하십시오."
wizard_restore="기존 데이터 사용"
wizard_create_new="설치 정리(구성 파일 및 데이터베이스를 포함한 모든 기존 데이터가 제거됩니다.)"
wizard_db_user_account_desc="데이터베이스 사용자 계정"
wizard_db_user_password_desc="데이터베이스 사용자 패스워드"
wizard_migrate_note="Joomla의 최신 버전은 MariaDB 10만 지원합니다.<br>시스템에서 기존 MariaDB 5 데이터베이스를 감지했습니다.<br>다음 단계에서는 데이터베이스가 MariaDB 5에서 MariaDB 10으로 마이그레이션됩니다.<br><br>설치를 진행하려면 MariaDB 5의 관리자 자격 증명을 입력하십시오."
wizard_migrate_title="Joomla 데이터베이스 마이그레이션"
wizard_remove_msql_title="Joomla 데이터베이스 제거"
wizard_remove_msql_desc="Joomla 데이터베이스가 제거되면 모든 데이터가 삭제됩니다."
wizard_msql_password_desc_remove="Joomla 데이터베이스를 제거하려면 데이터베이스 관리자 자격 증명을 입력하십시오."
wizard_admin_acc="계정"
wizard_admin_pass="패스워드"
wizard_unable_to_upgrade_title="이 버전으로 업데이트할 수 없음"
wizard_upgrade_from_prev_release_required_desc="패키지를 이 버전으로 업데이트하기 전에 <a href=\"{1}\" target=\"_blank\">이전 버전</a>으로 업데이트하십시오."
PageM5()
{
cat << EOF
{
	"step_title": "$wizard_migrate_title",
	"items": [{
		"type": "textfield",
		"desc": "$wizard_migrate_note",
		"subitems": [{
			"key": "wizard_m5_acc",
			"desc": "$wizard_admin_acc",
			"defaultValue": "root",
			"validator": {
				"allowBlank": false
			}
		}]
	}, {
		"type": "password",
		"subitems": [{
			"indent": 1,
			"key": "wizard_m5_pass",
			"desc": "$wizard_admin_pass"
		}]
	}]
}
EOF
}

PageRestore()
{
cat << EOF
{
	"step_title": "$wizard_found_backup",
	"items": [{
		"type": "singleselect",
		"desc": "$wizard_decide_restore",
		"subitems": [{
			"key": "restore_backup",
			"desc": "$wizard_restore",
			"defaultValue": true
		}, {
			"desc": "$wizard_create_new",
			"defaultValue": false
		}]
	}]
}
EOF
}

PageM10()
{
cat << EOF
{
	"step_title": "$wizard_set_data_title",
	"items": [{
		"type": "textfield",
		"desc": "$wizard_m10_desc",
		"subitems": [{
			"key": "wizard_m10_acc",
			"desc": "$wizard_admin_acc",
			"defaultValue": "root",
			"validator": {
				"allowBlank": false
			}
		}]
	}, {
		"type": "password",
		"subitems": [{
			"indent": 1,
			"key": "wizard_m10_pass",
			"desc": "$wizard_admin_pass"
		}]
	}]
}
EOF
}
CheckRestore()
{
cat << EOF
// find constructor contains restore page
for (i = arguments[0].ownerCt.items.length-1; i >= 1; i--){
	page = arguments[0].ownerCt.items.items[i];
	if (page.headline == \"${wizard_found_backup}\"){
		// check whether user wants to restore or not
		restore = page.items.items[1].checked;
		break;
	}
}
EOF
}

FindObj()
{
cat << EOF
for (i = 0; i < arguments[0].items.length; i++) {
	item = arguments[0].items.items[i]
	if (\"${1}\" == item.itemId){
		$2 = arguments[0].items.items[i];
		break;
	}
}
EOF
}

Activate()
{
cat << EOF
	"activeate": "{
		restore = $DEFAULT_RESTORE; // user restore or not, set by is_restore
		// will set restore
		$(CheckRestore)

		$(CustomPageActivate)

		if (arguments[0].headline == \"${wizard_db_settings}\") {
			create_db_multiselect = null;
			need_migrate_multiselect = null;
			old_db_textfield= null;
			drop_db_multiselect= null;
			new_db_textfield = null;
			db_user_textfield = null;
			grant_user_multiselect = null;
			mariadb_ver_textfield = null;

			$(FindObj "create_db_flag" "create_db_multiselect")
			$(FindObj "need_migrate" "need_migrate_multiselect")
			$(FindObj "pkgwizard_db_name" "new_db_textfield")
			$(FindObj "old_db_name" "old_db_textfield")
			$(FindObj "drop_db_flag" "drop_db_multiselect")
			$(FindObj "pkgwizard_db_user_account" "db_user_textfield")
			$(FindObj "grant_user_flag" "grant_user_multiselect")
			$(FindObj "mariadb_ver" "mariadb_ver_textfield")

			old_db = \"@OLD_DB@\";

			if (restore) {
				db_name = \"@OLD_DB@\";
				db_user = \"@OLD_USER@\";
			}
			else {
				db_name = \"$INIT_DB_NAME\";
				db_user = \"$INIT_DB_USER\";
			}

			old_db_textfield.setValue(old_db);
			new_db_textfield.setValue(db_name);
			new_db_textfield.setDisabled(restore);
			db_user_textfield.setValue(db_user);
			mariadb_ver_textfield.setValue(\"$MARIADB_VER\");

			need_migrate_multiselect.setVisible(false);
			create_db_multiselect.setVisible(false);
			drop_db_multiselect.setVisible(false);
			grant_user_multiselect.setVisible(false);

		}

	}",
EOF
}

Deactivate()
{
cat << EOF
	"deactivate": "{
		restore = $DEFAULT_RESTORE;
		$(CheckRestore) // if no radio button, will not set restore value
		if (arguments[0].headline == \"${wizard_db_settings}\") {
			create_db_multiselect = null;
			need_migrate_multiselect = null;
			create_db_collision_textfield = null;
			old_db_textfield= null;
			drop_db_multiselect= null;
			new_db_textfield = null;
			grant_user_multiselect = null;

			$(FindObj "create_db_flag" "create_db_multiselect")
			$(FindObj "need_migrate" "need_migrate_multiselect")
			$(FindObj "create_db_collision" "create_db_collision_textfield")
			$(FindObj "old_db_name" "old_db_textfield")
			$(FindObj "drop_db_flag" "drop_db_multiselect")
			$(FindObj "pkgwizard_db_name" "new_db_textfield")
			$(FindObj "grant_user_flag" "grant_user_multiselect")

			table = {
				\"5to10\": {\"migrate\": true, \"create\": false, \"grant_user\": true, \"drop_db\": true, \"create_db_collision\": \"error\"},
				\"new5to10\": {\"migrate\": false, \"create\": true, \"grant_user\": true, \"drop_db\": true, \"create_db_collision\": \"error\"},
				\"restore10\": {\"migrate\": false, \"create\": true, \"grant_user\": true, \"drop_db\": false, \"create_db_collision\": \"skip\"},
				\"new10\": {\"migrate\": false, \"create\": true, \"grant_user\": true, \"drop_db\": old_db_textfield.rawValue && (old_db_textfield.rawValue !== new_db_textfield.rawValue)? true : false, \"create_db_collision\": old_db_textfield.rawValue === new_db_textfield.rawValue? \"replace\" : \"error\"}
			};

			if (restore) {
				if ($NEED_MIGRATE) {
					state = \"5to10\";
				}
				else {
					state = \"restore10\";
				}
			}
			else {
				if ($NEED_MIGRATE) {
					state = \"new5to10\";
				}
				else {
					state = \"new10\";
				}
			}

			need_migrate_multiselect.setValue(table[state][\"migrate\"]);
			create_db_multiselect.setValue(table[state][\"create\"]);
			grant_user_multiselect.setValue(table[state][\"grant_user\"]);
			drop_db_multiselect.setValue(table[state][\"drop_db\"]);
			create_db_collision_textfield.setValue(table[state][\"create_db_collision\"]);
		}
}",
EOF
}

ApplyDBInfo()
{
	local page_db="$1"

	sed "s/@OLD_DB@/$DB_NAME/g;s/@OLD_USER@/$DB_USER/g" <<< "$page_db"
}

PageDB()
{
	local page_db=$(cat << EOF
{
	"step_title": "$wizard_db_settings",
	$(Activate)
	$(Deactivate)
	"items": [{
		"type": "textfield",
		"desc": "$wizard_set_db_desc",
		"subitems": [{
			"key": "pkgwizard_db_name",
			"desc": "$wizard_db_name_label",
			"disabled": true,
			"validator": {
				"allowBlank": false
			}
		}]
	}, {
		"type": "textfield",
		"subitems": [{
			"indent": 1,
			"key": "pkgwizard_db_user_account",
			"desc": "$wizard_db_user_account_desc",
			"validator": {
				"allowBlank": false
			}
		}]
	}, {
		"type": "password",
		"subitems": [{
			"indent": 1,
			"key": "pkgwizard_db_user_password",
			"desc": "$wizard_db_user_password_desc"
		}]
	}, {
		"type": "textfield",
		"subitems": [{
			"key": "create_db_collision",
			"desc": "drop or skip",
			"defaultValue": "skip",
			"hidden": true
		}]
	}, {
		"type": "multiselect",
		"subitems": [{
			"key": "need_migrate",
			"desc": "migrate or not",
			"hidden": true
		}]
	}, {
		"type": "multiselect",
		"subitems": [{
			"key": "create_db_flag",
			"desc": "create db or not",
			"hidden": true
		}]
	}, {
		"type": "textfield",
		"subitems": [{
			"key": "old_db_name",
			"desc": "old db name",
			"hidden": true
		}]
	}, {
		"type": "multiselect",
		"subitems": [{
			"key": "drop_db_flag",
			"desc": "drop old db or not",
			"hidden": true
		}]
	}, {
		"type": "multiselect",
		"subitems": [{
			"key": "grant_user_flag",
			"desc": "must grant user if exist wizard",
			"defaultVaule": true,
			"hidden": true
		}]
	}, {
		"type": "textfield",
		"subitems": [{
			"key": "mariadb_ver",
			"desc": "mariadb version",
			"defaultVaule": "$MARIADB_VER",
			"hidden": true
		}]
	}]
}
EOF
)
	ApplyDBInfo "$page_db"
}
main()
{
	local install_page=""

	CustomSetPages # set: MIGRATE_PAGES RESTORE_PAGES NEW_INSTALL_PAGES
	CustomSetBackupPath # set BACKUP_PATH
	CustomMainInstall

	DEFAULT_RESTORE=false
	NEED_MIGRATE=false
	MARIADB_VER="m10"
	DB_NAME=$(CustomGetDBName "$BACKUP_PATH")
	DB_USER=$(CustomGetDBUser "$BACKUP_PATH")

	if CustomHasRunWebsiteSetup "$BACKUP_PATH"; then
		local version="$(CustomGetValueFromMetaFile "version" | cut -s -d- -f2)"
		if NeedMigrateDB "$version"; then # check if needing to migrate MariaDB 5
			NEED_MIGRATE=true
			MARIADB_VER="m5"
			install_page="$MIGRATE_PAGES"
		else
			install_page="$RESTORE_PAGES"
		fi
	else
		install_page="$NEW_INSTALL_PAGES"
	fi

	echo "[$install_page]" > "${SYNOPKG_TEMP_LOGFILE}"

	return 0
}

main "$@"
main()
{
	local install_page=""

	CustomSetPages # set: MIGRATE_PAGES RESTORE_PAGES NEW_INSTALL_PAGES
	CustomSetBackupPath # set BACKUP_PATH
	CustomMainInstall

	DEFAULT_RESTORE=false
	NEED_MIGRATE=false
	MARIADB_VER="m10"
	DB_NAME=$(CustomGetDBName "$BACKUP_PATH")
	DB_USER=$(CustomGetDBUser "$BACKUP_PATH")

	if CustomHasRunWebsiteSetup "$BACKUP_PATH"; then
		local version="$(CustomGetValueFromMetaFile "version" | cut -s -d- -f2)"
		if NeedMigrateDB "$version"; then # check if needing to migrate MariaDB 5
			NEED_MIGRATE=true
			MARIADB_VER="m5"
			install_page="$MIGRATE_PAGES"
		else
			install_page="$RESTORE_PAGES"
		fi
	else
		install_page="$NEW_INSTALL_PAGES"
	fi

	echo "[$install_page]" > "${SYNOPKG_TEMP_LOGFILE}"

	return 0
}

main "$@"
