#!/bin/bash

# define main function
MAIN_INSTALL="main_install"
MAIN_UPGRADE="main_upgrade"
MAIN_UNINST="main_uninst"
# define each page's name
PAGE_MARIADB5="page_mariadb5"
PAGE_MARIADB10="page_mariadb10"
PAGE_RESTORE="page_restore"
PAGE_UPGRADE_RESTRICTED="page_upgrade_restricted"
PAGE_DB="page_db"
PAGE_UNINST="page_uninst"
DEFAULT="enu"
# set out file name
OUTPUT_INSTALL="install_uifile"
OUTPUT_UPGRADE="upgrade_uifile"
OUTPUT_UNINST="uninstall_uifile"
INIT_DB_NAME="" # need to be overwrited
INIT_DB_USER="" # need to be overwrited
INFO_FILE="" # need to be overwrited
BACKUP_PATH="" # need to be overwrited
# here to define what page is necessary for each stage: install, upgrade, and uninstall
PAGE_INST="$PAGE_MARIADB5 $PAGE_RESTORE $PAGE_MARIADB10 $PAGE_DB $MAIN_INSTALL"
PAGE_UPGRADE="$PAGE_UPGRADE_RESTRICTED $PAGE_MARIADB5 $PAGE_MARIADB10 $PAGE_DB $MAIN_UPGRADE"
PAGE_UNINST="$PAGE_UNINST $MAIN_UNINST"
MIGRATE_DB_VERSION="-1"

MainCreate()
{
	local output="$1"
	local pages="$2"
	local main_function="$3"
	local lan
	for dir in lang/*; do
		lan=$(basename "$dir")
		cat "wizard_common" "wizard_customized" > "${output}_${lan}.sh"
		CustomWizCreate "${output}_${lan}.sh"
		cat "lang/$lan" $pages "$main_function" >> "${output}_${lan}.sh"
	done
	chmod +x "${output}"*.sh
	cp -a "${output}_${DEFAULT}.sh" "${output}.sh"
}

IsUpgradable()
{
	true
}

NeedMigrateDB()
{
	local version="$1"
	if [ -z "$version" ] || [ "$version" -le "$MIGRATE_DB_VERSION" ]; then
		return 0
	fi
	return 1
}

CustomHasRunWebsiteSetup()
{
	local path="$1"
	[ -s "$path/$CONF_FILE" ]
}

CustomWizCreate()
{
	:
}

CustomSetBackupPath()
{
	BACKUP_PATH="$(CustomGetBackupPkgPath)"
}

CustomMainInstall()
{
	:
}

CustomGetValueFromMetaFile()
{
	local key="$1"
	local backup_info="/var/packages/.$PKG_DIR.conf"
	local syno_conf="/var/packages/$PKG_NAME/synology.conf"
	local record_file=""

	if [ -s "$INFO_FILE" ]; then
		record_file="$INFO_FILE"
	elif [ -s "$backup_info" ]; then
		record_file="$backup_info"
	elif [ -s "$syno_conf" ]; then
		record_file="$syno_conf"
	fi
	get_key_value "$record_file" "$key"
}

GetPreviousPkgBuildNumber()
{
	local old_pkg_ver="$SYNOPKG_OLD_PKGVER"
	if [ -z "${old_pkg_ver}" ]; then
		# SYNOPKG_OLD_PKGVER does not exist in un-install stage of upgrade
		old_pkg_ver="$SYNOPKG_PKGVER"
	fi

	local old_build_number=$(echo "$old_pkg_ver" | cut -d- -f2)
	echo "$old_build_number"
}

CustomGetBackupPrefix()
{
	local backup_prefix="/$(readlink /var/services/web | cut -d/ -f2)/@appstore/.$PKG_DIR"
	echo "$backup_prefix"
}

CustomGetBackupPkgPath()
{
	local backup_pkg_path="$(CustomGetBackupPrefix)/$PKG_DIR"
	echo "$backup_pkg_path"
}

# we define pages in fuction not in global variables,
# because functions in wizard steps will be merged after calling MainCreate.
CustomSetPages()
{
	NEW_INSTALL_PAGES="$(PageM10),$(PageDB)"
	RESTORE_PAGES="$(PageRestore),$(PageM10),$(PageDB)"
	MIGRATE_PAGES="$(PageM5),$(PageRestore),$(PageM10),$(PageDB)"
}

CustomParseDBConf()
{
	:
}

CustomGetDBName()
{
	:
}

CustomGetDBUser()
{
	:
}

CustomGetDBPass()
{
	:
}

CustomPageActivate()
{
	:
}

quote_json() {
	sed -e 's|\\|\\\\|g' -e 's|\"|\\\"|g'
}

#!/bin/bash
# Copyright (c) 2000-2017 Synology Inc. All rights reserved.

PKG_NAME="Joomla"
PKG_DIR="joomla"
WEBSITE_ROOT="/var/services/web/$PKG_DIR"
CONF_FILE="configuration.php"
MIGRATE_DB_VERSION="0154"
INIT_DB_NAME="joomla"
INIT_DB_USER="joomla_user"
INFO_FILE="/usr/syno/etc/packages/$PKG_NAME/$PKG_DIR.conf"


CustomGetBackupPkgPath()
{
	local backup_prefix=$(CustomGetBackupPrefix)
	local backup_pkg_path="$(CustomGetBackupPrefix)/joomla"
	if [ ! -d "$backup_pkg_path" ]; then
		backup_pkg_path=$(dirname "$(ls -t "$backup_prefix"/*/configuration.php | head -1)")
	fi
	echo "$backup_pkg_path"
}

CustomParseDBConf()
{
	local path="$1"
	local db_info="$2"
	local grep_info="$3"
	local info_output=$(CustomGetValueFromMetaFile "$db_info")

	if [ -z "$info_output" ] && [ -s "$path/$CONF_FILE" ]; then
		info_output=$(grep "public \$$grep_info =" "$path/$CONF_FILE" | cut -d\' -f2)
	fi

	echo "$info_output"
}

CustomGetDBName()
{
	CustomParseDBConf "$1" "db_name" "db"
}

CustomGetDBUser()
{
	CustomParseDBConf "$1" "db_user" "user"
}

CustomGetDBPass()
{
	CustomParseDBConf "$1" "db_pass" "password"
}
wizard_db_settings="Joomla-Datenbank einrichten"
wizard_db_name_desc="Bitte geben Sie den Datenbanknamen für die Joomla Datenbank ein."
wizard_db_name_label="Datenbankname"
wizard_m10_desc="Bitte geben Sie die Administrator-Zugriffsdaten für MariaDB 10 ein, um mit den Joomla-Datenbankeinstellungen fortzufahren."
wizard_admin_acc="Konto"
wizard_admin_pass="Kennwort"
wizard_set_db_desc="Bitte erstellen Sie ein neues exklusives Datenbankkonto für Joomla, um vorhandene/neue Daten zu verwenden."
wizard_set_data_title="Joomla einrichten"
wizard_found_backup="Bitte wählen Sie eine Methode, um Daten zu importieren."
wizard_decide_restore="Joomla-Datenbank ist bereits vorhanden.<br>Bitte wählen Sie eine der folgenden Aktionen aus:"
wizard_restore="Vorhandene Daten verwenden"
wizard_create_new="Installation reinigen (sämtliche vorhandenen Daten, einschließlich Konfigurationsdateien und die Datenbank, werden entfernt.)"
wizard_db_user_account_desc="Datenbank-Benutzerkonto"
wizard_db_user_password_desc="Datenbankbenutzer-Kennwort"
wizard_migrate_note="Die neueste Version von Joomla unterstützt nur MariaDB 10.<br>Das System hat eine vorhandene MariaDB 5-Datenbank erkannt.<br>Im nächsten Schritt wird Ihre Datenbank von MariaDB 5 zu MariaDB 10 migriert.<br><br>Bitte geben Sie die admin-Zugriffsdaten von MariaDB 5 ein, um die Installation fortzusetzen."
wizard_migrate_title="Joomla-Datenbank migrieren"
wizard_remove_msql_title="Joomla-Datenbank entfernen"
wizard_remove_msql_desc="Wenn die Joomla-Datenbank entfernt wird, werden sämtliche Daten gelöscht."
wizard_msql_password_desc_remove="Geben Sie die Anmeldeinformationen des Datenbank-Administrators ein, um die Joomla-Datenbank zu entfernen."
wizard_admin_acc="Konto"
wizard_admin_pass="Kennwort"
wizard_unable_to_upgrade_title="Aktualisierung auf diese Version nicht möglich"
wizard_upgrade_from_prev_release_required_desc="Aktualisieren Sie das Paket vor dem Update auf diese Version bitte auf die <a href=\"{1}\" target=\"_blank\">Vorgängerversion</a>."
PageUninst()
{
cat << EOF
{
	"step_title": "$wizard_remove_msql_title",
	"items": [{
		"type": "multiselect",
		"desc": "$wizard_remove_msql_desc",
		"subitems": [{
			"key": "pkgwizard_remove_mysql",
			"desc": "$wizard_remove_msql_title"
		}]
	}, {
		"type": "textfield",
		"desc": "$wizard_msql_password_desc_remove",
		"subitems": [{
			"key": "wizard_m10_acc",
			"desc": "$wizard_admin_acc",
			"defaultValue": "root"
		}]
	}, {
		"type": "password",
		"subitems": [{
			"indent": 1,
			"key": "wizard_m10_pass",
			"desc": "$wizard_admin_pass"
		}]
	}]
}
EOF
}
main()
{
	local uninst_page="$(PageUninst)"
	echo "[$uninst_page]" > "${SYNOPKG_TEMP_LOGFILE}"

	return 0
}

main "$@"
main()
{
	local uninst_page="$(PageUninst)"
	echo "[$uninst_page]" > "${SYNOPKG_TEMP_LOGFILE}"

	return 0
}

main "$@"
